@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                            Header file for CDF distribution.
 *
@@ -48,6 +53,12 @@
 *                               bit mode, respectively on Mac.
 *   V2.16 28-Apr-09, M Liu      Added linux-PPC (pre-defined macro "__PPC__"
 *                               by gcc for either 32/64-bit).
+*   V2.17 16-Jul-12, M Liu      Changed "uInt" to "uInt4", "uLong" to "uLongx"
+*                               and "Byte" to "Byte1" so they won't be in
+*                               conflict with the same typedef's defined in
+*                               zlib.
+*   V2.18 09-Aug-13, M Liu      Added cdfid_AllocateMemory, cdfid_getCDFid,
+*                               cdfid_FreeMemory.
 *
 ******************************************************************************/
 
@@ -397,8 +408,8 @@
 typedef int Logical;
 
 typedef unsigned short uShort;
-typedef unsigned int uInt;
-typedef unsigned long uLong;
+typedef unsigned int uInt4;
+typedef unsigned long uLongx;
 
 #if defined(alphaosf) || defined(IRIX64bit) || defined(SOLARIS64) || \
     defined(__amd64) || defined(__x86_64__) || defined(__ia64__) || \
@@ -411,11 +422,11 @@ typedef unsigned long uLong;
    * `int' is 32 bits in these cases.
    ***************************************************************************/
    typedef int Int32;
-   typedef uInt uInt32;
+   typedef uInt4 uInt32;
    typedef long Int64;
 #else
    typedef long Int32;
-   typedef uLong uInt32;
+   typedef uLongx uInt32;
    typedef long long Int64;
 #endif
 
@@ -440,7 +451,7 @@ typedef unsigned char uChar;
   /****************************************************************************
   * `Byte' is already defined in `types.h' on a Macintosh (MPW C & Think C).
   ****************************************************************************/
-  typedef uChar Byte;
+  typedef uChar Byte1;
 #endif
 
 typedef uChar uByte;
@@ -458,7 +469,7 @@ typedef sChar sByte;
 #if defined(STAT)
 #  undef STAT
 #endif
-#define STAT stat
+/* #define STAT stat */
 #define FOPEN fopen
 #define FWRITE fwrite
 #define FREAD fread
@@ -476,6 +487,7 @@ typedef sChar sByte;
 #    define FREAD64 _read
 #    define FCLOSE64 _close
 #    define FLUSH64 _commit
+#    define STAT    _stat64
 #  else
 #    if defined(__MINGW32__) || defined(AIX)
        typedef off64_t OFF_T;
@@ -503,12 +515,18 @@ typedef sChar sByte;
 #      define FOPEN64 fopen64
 #    endif
 #    define FWRITE64 fwrite
-#    define FREAD64 fread
+#    define FREAD64  fread
 #    define FCLOSE64 fclose
-#    define FLUSH64 fflush
+#    define FLUSH64  fflush
+#    if defined(__CYGWIN__) || defined(vms)
+#      define STAT     stat
+#    else
+#      define STAT     stat64
+#    endif
 #  endif
 #else
    typedef long OFF_T;
+#  define STAT  stat
 #endif
 
 /******************************************************************************
@@ -525,7 +543,7 @@ typedef sChar sByte;
 * Debugging.
 ******************************************************************************/
 
-#define Trace(msg) {printf(msg); fflush(stdout); fflush(stderr);}
+#define TraceZ(msg) {printf(msg); fflush(stdout); fflush(stderr);}
 
 /******************************************************************************
 * Variable argument list function prototyping.
@@ -616,7 +634,6 @@ ap = _apT_; \
 #define NUM_HUFF_PARMS		1
 #define NUM_AHUFF_PARMS		1
 #define NUM_GZIP_PARMS		1
-/* #define NUM_ZLIB_PARMS          1 */
 
 /*****************************************************************************
 * ASCII characters.
@@ -922,10 +939,14 @@ typedef struct vSTATSstruct {
 #define VAR_ALLOCATEDTO(zOp) BOO(zOp,zVAR_ALLOCATEDTO_,rVAR_ALLOCATEDTO_)
 #define VAR_nINDEXLEVELS(zOp) BOO(zOp,zVAR_nINDEXLEVELS_,rVAR_nINDEXLEVELS_)
 #define VAR_RECORDS(zOp) BOO(zOp,zVAR_RECORDS_,rVAR_RECORDS_)
+#define VAR_RECORDS_RENUMBER(zOp) BOO(zOp,zVAR_RECORDS_RENUMBER_,rVAR_RECORDS_RENUMBER_)
 #define VAR_BLOCKINGFACTOR(zOp) BOO(zOp,zVAR_BLOCKINGFACTOR_, \
 					rVAR_BLOCKINGFACTOR_)
 #define VAR_RESERVEPERCENT(zOp) BOO(zOp,zVAR_RESERVEPERCENT_, \
 					rVAR_RESERVEPERCENT_)
+#define VAR_RECNUMBER(zOp) BOO(zOp,zVAR_RECNUMBER_,rVARs_RECNUMBER_)
+#define VAR_RECCOUNT(zOp) BOO(zOp,zVAR_RECCOUNT_,rVARs_RECCOUNT_)
+#define VAR_RECINTERVAL(zOp) BOO(zOp,zVAR_RECINTERVAL_,rVARs_RECINTERVAL_)
 
 #define VARs_MAXREC(zOp) BOO(zOp,zVARs_MAXREC_,rVARs_MAXREC_)
 #define VARs_RECDATA(zOp) BOO(zOp,zVARs_RECDATA_,rVARs_RECDATA_)
@@ -955,6 +976,15 @@ VISIBLE_PREFIX int cdf_FreeMemoryX PROTOARGs((
 VISIBLE_PREFIX void *CallocateMemory PROTOARGs((
   size_t nObjects, size_t objSize, void (*fatalFnc) PROTOARGs((char *msg))
 ));
+VISIBLE_PREFIX void cdfid_AllocateMemory PROTOARGs((
+  void *ptr, void (*fatalFnc) PROTOARGs((char *msg))
+));
+VISIBLE_PREFIX int cdfid_FreeMemory PROTOARGs((
+  void *ptr, void (*fatalFnc) PROTOARGs((char *msg))
+));
+VISIBLE_PREFIX CDFid cdfid_getCDFid PROTOARGs((
+  Int32 id, void (*fatalFnc) PROTOARGs((char *msg))
+));
 VISIBLE_PREFIX char *strcpyX PROTOARGs((char *dst, char *src, size_t max));
 VISIBLE_PREFIX char *strcatX PROTOARGs((char *dst, char *src, size_t max));
 #if defined(__cplusplus)
@@ -977,12 +1007,6 @@ VISIBLE_PREFIX Logical NegativeZeroReal8 PROTOARGs((double *value));
 VISIBLE_PREFIX Logical NegativeZeroReal4 PROTOARGs((float *value));
 VISIBLE_PREFIX void MakeUpperString PROTOARGs((char *string));
 VISIBLE_PREFIX void MakeLowerString PROTOARGs((char *string));
-VISIBLE_PREFIX void ROWtoCOL PROTOARGs((
-  void *iBuffer, void *oBuffer, long numDims, long dimSizes[], long nValueBytes
-));
-VISIBLE_PREFIX void COLtoROW PROTOARGs((
-  void *iBuffer, void *oBuffer, long numDims, long dimSizes[], long nValueBytes
-));
 VISIBLE_PREFIX void INCRindicesROW PROTOARGs((
   long numDims, long *dimSizes, long *indices
 ));
