@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF					      Header for CDF toolkit.
 *
@@ -159,7 +164,7 @@
 
 #define QOP_MAX_ARGVs	40
 #define QOP_MAX_PARMs	40
-#define QOP_MAX_QUALs	35
+#define QOP_MAX_QUALs	40
 
 /*****************************************************************************
 * QOP structure.
@@ -641,6 +646,7 @@ extern int lastSecond;
 * Function prototypes.
 *****************************************************************************/
 
+int UTF8StrLength PROTOARGs((unsigned char *));
 void ASSIGNx PROTOARGs((void *, void *, long, long));
 Logical EQx PROTOARGs((void *, void *, long, long));
 Logical NEx PROTOARGs((void *, void *, long, long));
@@ -663,6 +669,8 @@ void EncodeRecordIndicesJustify PROTOARGs((
 ));
 void Justify PROTOARGs((char *string, int minWidth));
 void RemoveLeadingBlanks PROTOARGs((char *string));
+void RemoveLeading0 PROTOARGs((char *string, size_t len));
+int ISTPname PROTOARGs((char *name));
 void RemoveTrailingBlanks PROTOARGs((char *string));
 void RemoveWhiteSpace PROTOARGs((char *string));
 void CopyToField PROTOARGs((char *string, char *field, int length));
@@ -733,7 +741,7 @@ char *strstrIgCase PROTOARGs((char *string, char *substring));
 char *strchrIgCase PROTOARGs((char *str, int chr));
 int strcmpIgCase PROTOARGs((char *string1, char *string2));
 int strncmpIgCase PROTOARGs((char *string1, char *string2, size_t count));
-int strncmpIgCasePattern PROTOARGs((char *string1, char *string2, 
+int strncmpIgCasePattern PROTOARGs((char *string, char *pattern, 
   size_t count
 ));
 void OutputWithMargin PROTOARGs((
@@ -758,6 +766,15 @@ Logical S3qualifierLong PROTOARGs((
   QOP *qop, long *variable, int S1x, long S1, int S2x, long S2, int S3x,
   long S3, long defaultS, char *conflictText
 ));
+Logical S4qualifierLong PROTOARGs((
+  QOP *qop, long *variable, int S1x, long S1, int S2x, long S2, int S3x,
+  long S3, int S4x, long S4, long defaultS, char *conflictText
+));
+Logical S6qualifierLong PROTOARGs((
+  QOP *qop, long *variable, int S1x, long S1, int S2x, long S2, int S3x,
+  long S3, int S4x, long S4, int S5x, long S5, int S6x, long S6, 
+  long defaultS, char *conflictText
+));
 void BlankPadRight PROTOARGs((char *string, int count));
 CDFstatus GetFormatEntry PROTOARGs((Logical, long, char **));
 Logical ParseOptionList PROTOARGs((
@@ -797,7 +814,7 @@ void IncrIndicesFirstLastCol PROTOARGs((
 ));
 Logical AllocateBuffers PROTOARGs((
   long nRecords, long numDims, long dimSizes[], struct GroupStruct *groups,
-  int nScalarBuffers, int nHyperBuffers, Byte **handles[],
+  int nScalarBuffers, int nHyperBuffers, Byte1 **handles[],
   size_t nValueBytes[], Logical rowMajor, int minNofHypers,
   void (*fatalFnc) PROTOARGs((char *msg))
 ));
@@ -820,7 +837,7 @@ Logical HyperStartOfRecord PROTOARGs((
 ));
 void WriteOutPct PROTOARGs((int pct));
 Logical SwitchMajority PROTOARGs((
-  Byte *buffer, Logical rowMajor, long numDims, long dimSizes[],
+  Byte1 *buffer, Logical rowMajor, long numDims, long dimSizes[],
   long recCount, size_t nValueBytes
 ));
 Logical CompressedCDF PROTOARGs((char *CDFpath));
@@ -847,6 +864,12 @@ int GetNumTokens PROTOARGs((int token, char *string));
 Logical ValidateQual PROTOARGs((
   char *qual, char *validQuals[]
 ));
+int BinarySearch PROTOARGs((
+  long dataType, void *values, void *value, int min, int max
+));
+void MergeSort PROTOARGs((
+  long dataType, void *values, void *tmp, int left, int right
+));
 #if defined(mac)
 void MacExecuteSO PROTOARGs((
   Logical (*exeFnc)(int argC, char *argV[]),
@@ -924,6 +947,15 @@ void DebugMessage PROTOARGs((char *text));
 }
 #endif
 
+#if defined(win32)
+#define isnan(x) _isnan(x)
+#define isinf(x) (!_finite(x))
+#endif
+
+#if defined(vms)
+#define isinf(x) (!finite(x))
+#endif
+
 /*****************************************************************************/
 
 #endif
