@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                        CDF `create' operations.
 *
@@ -22,6 +27,7 @@
 *   V1.6   8-Apr-04, M Liu      Save the currently created variable's offset. 
 *   V1.7  13-Oct-06, M Liu      Changed to allow upper and lower case CDF  
 *                               name to be used on win32.
+*   V3.5  23-Jul-13, M Liu      Changed the alternative encoding to IBMPC.
 *
 ******************************************************************************/
 
@@ -63,6 +69,7 @@ switch (Va->item) {
     Logical no_append = FALSE;
     uInt32 magicNumber1 = V2magicNUMBER_1;
     uInt32 magicNumber2u = V2magicNUMBER_2u;
+    char *del = NULL;
     /**************************************************************************
     * Get arguments for this operation/item.
     **************************************************************************/
@@ -92,7 +99,25 @@ switch (Va->item) {
     MakeUpperString (CDFnameT);
 #endif
     RemoveCDFFileExtension(CDFnameT, CDFnameTx);
+#if defined(unix) || defined(win32) || defined(dos)
+# if defined(unix)
+    del = strrchr(CDFnameTx, '/');
+# else
+    del = strrchr(CDFnameTx, '\\');
+    if (del == NULL) {
+#     if defined(win32)
+        del = strrchr(CDFnameTx, '/');
+#     endif
+    }
+# endif
+    if (del == NULL) {
+      if (!ValidCDFname(CDFnameTx)) return BAD_CDF_NAME;
+    } else {
+      if (!ValidCDFname(del+1)) return BAD_CDF_NAME;
+    }
+#else
     if (!ValidCDFname(CDFnameTx)) return BAD_CDF_NAME;
+#endif
     BuildFilePath (CDFt, CDFnameTx, no_append, upper_case_ext, version_numbers,
 		   INT32_ZERO, CDFnameX);
     if (IsReg(CDFnameX)) return CDF_EXISTS;
@@ -104,13 +129,13 @@ switch (Va->item) {
     /**************************************************************************
     * Allocate/initialize CDF structure.
     **************************************************************************/
-    CDF = (struct CDFstruct *) cdf_AllocateMemory (sizeof(struct CDFstruct), NULL);
+    CDF = (struct CDFstruct *) cdf_AllocateMemory ((size_t)sizeof(struct CDFstruct), NULL);
     if (CDF == NULL) {
       V_close (dotFp, NULL, NULL);
       DeleteFile (CDFnameX);
       return BAD_MALLOC;
     }
-    CDF->CDFname = (char *) cdf_AllocateMemory (strlen(CDFnameTx) + 1, NULL);
+    CDF->CDFname = (char *) cdf_AllocateMemory ((size_t)strlen(CDFnameTx) + 1, NULL);
     if (CDF->CDFname == NULL) {
       V_close (dotFp, NULL, NULL);
       DeleteFile (CDFnameX);
@@ -147,7 +172,7 @@ switch (Va->item) {
     CDF->rVars = NULL;
     CDF->zVars = NULL;
     CDF->encoding = BOO(DEFAULT_TO_HOST_ENCODING,
-			HostEncoding(),NETWORK_ENCODING);
+			HostEncoding(),IBMPC_ENCODING);
     CDF->rowMajor = DEFAULT_TO_ROW_MAJOR;
     CDF->singleFile = DEFAULT_TO_SINGLE_FILE;
     CDF->rMaxRec = NO_RECORD;
@@ -276,6 +301,8 @@ switch (Va->item) {
     Int32 offset, ntlOffset, VDRhead, nVars;
     Int32 dataType, numElements;
     int dimN, *max;
+    Int32 padSize, toPad;
+    void *pad;
     name = va_arg (Va->ap, char *);
     dataType = (Int32) va_arg (Va->ap, long);
     numElements = (Int32) va_arg (Va->ap, long);
@@ -353,7 +380,7 @@ switch (Va->item) {
       if (*max > 0)
 	ptr = (struct VarStruct **) cdf_ReallocateMemory (*vars, nBytes, NULL);
       else
-	ptr = (struct VarStruct **) cdf_AllocateMemory (nBytes, NULL);
+	ptr = (struct VarStruct **) cdf_AllocateMemory ((size_t)nBytes, NULL);
       if (ptr == NULL) return BAD_MALLOC;
       *vars = ptr;
       for (varN = *max; varN < newMaxVars; varN++) (*vars)[varN] = NULL;
@@ -411,18 +438,25 @@ switch (Va->item) {
 	 dimN < numDims; dimN++) {
        VDR.DimVarys[dimN] = BOO(dimVariances[dimN],VARY,NOVARY);
     }
-    if (!sX(AllocateIR(CDF,VDR.RecordSize,&offset),&pStatus)) {
+    padSize = (int) (CDFelemSize(dataType) * numElements);
+    pad = (void *) cdf_AllocateMemory((size_t)padSize, NULL);
+    if (!sX(AllocateIR(CDF,VDR.RecordSize+padSize,&offset),&pStatus)) {
       DeleteFile (pathnameX);
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
     }
+    toPad = VDR.RecordSize;
+    DefaultPadValuePre350(dataType, numElements, pad);
+    VDR.RecordSize += padSize;
+    SetBit32 (&VDR.Flags,VDR_PADVALUE_BIT);
     if (!sX(WriteVDR(CDF,CDF->fp,offset,zOp,
-		     VDR_RECORD,&VDR,NULL,
+		     VDR_RECORD,&VDR,pad,
 		     VDR_NULL),&pStatus)) {
       DeleteFile (pathnameX);
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
     }
+    cdf_FreeMemory (pad, NULL);
     /**************************************************************************
     * Point next-to-last rVDR/zVDR (or GDR) to this rVDR/zVDR.
     **************************************************************************/
