@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                        CDF `delete' operations.
 *
@@ -26,6 +31,8 @@
 *   V2.4  24-Jan-11, M Liu      Modified UpdateIndexEntries_r_64 to acquire
 *                               physical record size to set VXR's last record.
 *   V2.5  09-May-12, M Liu      Remodified UpdateIndexEntries_r_64.
+*   V2.6  27-May-14, M Liu      Added renumbering the sparse records after
+*                               deletion.
 *
 ******************************************************************************/
 
@@ -40,7 +47,8 @@ static CDFstatus WasteTree_r_64 PROTOARGs((
   struct CDFstruct *CDF, OFF_T vxrOffset
 ));
 static CDFstatus DeleteVarRecords64 PROTOARGs((
-  struct CDFstruct *CDF, struct VarStruct *Var, Int32 firstRec, Int32 lastRec
+  struct CDFstruct *CDF, struct VarStruct *Var, Int32 firstRec, Int32 lastRec,
+  Logical reNumber
 ));
 static CDFstatus DeleteRecords64 PROTOARGs((
   struct CDFstruct *CDF, struct VarStruct *Var, Int32 firstRec, Int32 lastRec,
@@ -415,11 +423,15 @@ switch (Va->item) {
     break;
   }
   /****************************************************************************
-  * rVAR_RECORDS_/zVAR_RECORDS_, 
+  * rVAR_RECORDS_/zVAR_RECORDS_/rVAR_RECORDS_RENUMBER_/zVAR_RECORDS_RENUMBER_
   ****************************************************************************/
   case rVAR_RECORDS_:
-  case zVAR_RECORDS_: {
-    Logical zOp = (Va->item == zVAR_RECORDS_);
+  case zVAR_RECORDS_:
+  case rVAR_RECORDS_RENUMBER_:
+  case zVAR_RECORDS_RENUMBER_: {
+    Logical zOp = (Va->item == zVAR_RECORDS_ ||
+                   Va->item == zVAR_RECORDS_RENUMBER_);
+    Logical reNumber;
     struct CDFstruct *CDF; struct VarStruct *Var;
     Int32 firstRec = (Int32) va_arg (Va->ap, long);
     Int32 lastRec = (Int32) va_arg (Va->ap, long);
@@ -438,7 +450,12 @@ switch (Va->item) {
       AbortAccess64 (CDF, UPDATE, noDELETE);
       return pStatus;
     }
-    if (!sX(DeleteVarRecords64(CDF,Var,firstRec,lastRec),&pStatus)) {
+    if (Va->item == rVAR_RECORDS_RENUMBER_ ||
+        Va->item == zVAR_RECORDS_RENUMBER_)
+      reNumber = TRUE;
+    else
+      reNumber = FALSE;
+    if (!sX(DeleteVarRecords64(CDF,Var,firstRec,lastRec,reNumber),&pStatus)) {
       AbortAccess64 (CDF, UPDATE, noDELETE);
       return pStatus;
     }
@@ -722,11 +739,12 @@ OFF_T vxrOffset;
 * DeleteVarRecords64.
 ******************************************************************************/
 
-static CDFstatus DeleteVarRecords64 (CDF, Var, firstRec, lastRec)
+static CDFstatus DeleteVarRecords64 (CDF, Var, firstRec, lastRec, reNumber)
 struct CDFstruct *CDF;
 struct VarStruct *Var;
 Int32 firstRec;
 Int32 lastRec;
+Logical reNumber;
 {
   CDFstatus pStatus = CDF_OK; Int32 deletedTo, recNum; Logical found;
   /****************************************************************************
@@ -737,7 +755,7 @@ Int32 lastRec;
     * Standard variable in a single-file CDF.
     **************************************************************************/
     case STANDARD_: {
-      Int32 aboveRecord, deleteCount; Int32 adjust;
+      Int32 aboveRecord, deleteCount;
       OFF_T vxrHead;
       /************************************************************************
       * Verify/adjust the records to be deleted.
@@ -797,7 +815,9 @@ Int32 lastRec;
     /**************************************************************************
     * Sparse records.
     **************************************************************************/
-    case SPARSE_RECORDS_:
+    case SPARSE_RECORDS_: {
+      Int32 aboveRecord, deleteCount;
+      OFF_T vxrHead; long maxRec;
       /************************************************************************
       * Flush staging area first.  This will automatically clear the staging
       * area as well.
@@ -815,44 +835,99 @@ Int32 lastRec;
 	 if (!sX(DeleteRecords64(CDF,Var,recNum,
 			         lastRec,&deletedTo),&pStatus)) return pStatus;
       }
-      /************************************************************************
-      * Update the control information.
-      ************************************************************************/
-      if (INCLUSIVE(firstRec,Var->maxWritten,lastRec)) {
-	if (!sX(PrevRecord64(CDF,Var->VDRoffset64,Var->zVar,
-			     (Int32)(firstRec - 1),
-			     &(Var->maxWritten),&found),&pStatus)) return
-								 pStatus;
-	if (!found) Var->maxWritten = NO_RECORD;
-      }
-      if (INCLUSIVE(firstRec,Var->maxAllocated,lastRec)) {
-	if (!sX(PrevRecord64(CDF,Var->VDRoffset64,Var->zVar,
-			     (Int32)(firstRec - 1),
-			     &(Var->maxAllocated),&found),&pStatus)) return
-								   pStatus;
-	if (!found) Var->maxAllocated = NO_RECORD;
-      }
-      /************************************************************************
-      * Update the maximum record field in the VDR.
-      ************************************************************************/
-      if (INCLUSIVE(firstRec,Var->maxRec,lastRec)) {
-	if (!sX(PrevRecord64(CDF,Var->VDRoffset64,Var->zVar,
-			     (Int32)(firstRec - 1),
-			     &(Var->maxRec),&found),&pStatus)) return pStatus;
-	if (!found) Var->maxRec = NO_RECORD;
-	if (!sX(WriteVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
-			   VDR_MAXREC,&(Var->maxRec),
-			   VDR_NULL),&pStatus)) return pStatus;
+      if (reNumber) {
+        deleteCount = lastRec - firstRec + 1;
+        /*********************************************************************
+        * Update the index entries.
+        *********************************************************************/
+        aboveRecord = firstRec - 1;
+        if (!sX(ReadVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
+                          VDR_VXRHEAD,&vxrHead,
+                          VDR_NULL),&pStatus)) return pStatus;
+        if (!sX(UpdateIndexEntries_r_64(CDF->fp,vxrHead,aboveRecord,
+                                        deleteCount),&pStatus))
+          return pStatus;
+        /*********************************************************************
+        * Update the control information.
+        *********************************************************************/
+        if (firstRec <= Var->maxWritten) {
+          if (lastRec > Var->maxWritten)
+            Var->maxWritten = firstRec - 1;
+          else
+            Var->maxWritten -= deleteCount;
+        }
+        if (firstRec <= Var->maxAllocated) {
+          if (lastRec > Var->maxAllocated)
+            Var->maxAllocated = firstRec - 1;
+          else
+            Var->maxAllocated -= deleteCount;
+        }
+        /*********************************************************************
+        * Update the maximum record field in the VDR.
+        *********************************************************************/
+        maxRec = Var->maxRec;
+        if (firstRec <= Var->maxRec) {
+          if (lastRec > Var->maxRec)
+            maxRec = firstRec - 1;
+          else
+            maxRec -= deleteCount;
+          if (!sX(WriteVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
+                             VDR_MAXREC,&(maxRec),
+                             VDR_NULL),&pStatus)) return pStatus;
+        }
+        if (INCLUSIVE(firstRec,Var->maxRec,lastRec)) {
+  	  if (!sX(PrevRecord64(CDF,Var->VDRoffset64,Var->zVar,
+			       (Int32)(firstRec - 1),
+			       &(Var->maxRec),&found),&pStatus)) return
+                                                                     pStatus;
+    	  if (!found) Var->maxRec = NO_RECORD;
+	  if (!sX(WriteVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
+			     VDR_MAXREC,&(Var->maxRec),
+			     VDR_NULL),&pStatus)) return pStatus;
+        } else
+          Var->maxRec = maxRec;
+      } else {
+        /*********************************************************************
+        * Update the control information.
+        *********************************************************************/
+        if (INCLUSIVE(firstRec,Var->maxWritten,lastRec)) {
+          if (!sX(PrevRecord64(CDF,Var->VDRoffset64,Var->zVar,
+                               (Int32)(firstRec - 1),
+                               &(Var->maxWritten),&found),&pStatus)) return
+                                                                     pStatus;
+          if (!found) Var->maxWritten = NO_RECORD;
+        }
+        if (INCLUSIVE(firstRec,Var->maxAllocated,lastRec)) {
+          if (!sX(PrevRecord64(CDF,Var->VDRoffset64,Var->zVar,
+                               (Int32)(firstRec - 1),
+                               &(Var->maxAllocated),&found),&pStatus)) return
+                                                                     pStatus;
+          if (!found) Var->maxAllocated = NO_RECORD;
+        }
+        /*********************************************************************
+        * Update the maximum record field in the VDR.
+        *********************************************************************/
+        if (INCLUSIVE(firstRec,Var->maxRec,lastRec)) {
+          if (!sX(PrevRecord64(CDF,Var->VDRoffset64,Var->zVar,
+                               (Int32)(firstRec - 1),
+                               &(Var->maxRec),&found),&pStatus)) return
+                                                                     pStatus;
+          if (!found) Var->maxRec = NO_RECORD;
+          if (!sX(WriteVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
+                             VDR_MAXREC,&(Var->maxRec),
+                             VDR_NULL),&pStatus)) return pStatus;
+        }
       }
       Var->firstRecInVVR = NO_RECORD;
       Var->lastRecInVVR = NO_RECORD;
       Var->offsetOfVVR64 = (OFF_T) NO_OFFSET64;
       break;
+    }
     /**************************************************************************
     * Compressed records.
     **************************************************************************/
     case COMPRESSED_: {
-      Int32 aboveRecord, deleteCount; Int32 adjust;
+      Int32 aboveRecord, deleteCount;
       OFF_T vxrHead;
       /************************************************************************
       * Flush/clear staging area first.
