@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                       CDF `get' operations.
 *
@@ -63,7 +68,7 @@ switch (Va->item) {
 #if defined(vms) || defined(dos)
     MakeUpperString (CDFpathT);
 #endif
-    if (!ValidCDFname(CDFpathT)) return BAD_CDF_NAME;
+/*  if (!ValidCDFname(CDFpathT)) return BAD_CDF_NAME; */
     if (!sX(FindCDF(CDFpathT,&no_append,
 		    &upper_case_ext,
 		    &version_numbers),&pStatus)) return pStatus;
@@ -278,7 +283,7 @@ switch (Va->item) {
     long *cParms = va_arg (Va->ap, long *);     /* Compression parameters. */
     long *cPct = va_arg (Va->ap, long *);	/* Compression percentage. */
     struct CDFstruct *CDF; struct CCRstruct CCR; struct CPRstruct CPR;
-    int i; long cTotal, uTotal;
+    int i; Int32 cTotal, uTotal;
     SelectCDF (Cur->cdf, CDF)
     /**************************************************************************
     * If multi-file or uncompressed...
@@ -309,7 +314,7 @@ switch (Va->item) {
     else {
       cTotal = MAGIC_NUMBERS_SIZE + CCR.RecordSize + CPR.RecordSize;
       uTotal = MAGIC_NUMBERS_SIZE + CCR.uSize;
-      *cPct = (100L * cTotal) / uTotal;
+      *cPct = (long) (((100.0 * cTotal) / uTotal) + 0.5);
     }
     break;
   }
@@ -1356,7 +1361,7 @@ switch (Va->item) {
       }
     }
     else {
-      DefaultPadValue (dataType, numElems, padValue);
+      DefaultPadValuePre350 (dataType, numElems, padValue);
       if (!sX(ConvertBuffer(HostEncoding(),CDF->decoding,
     		            CDF->negToPosFp0,dataType,
 			    numElems,padValue,padValue),&pStatus)) {
@@ -1490,7 +1495,7 @@ switch (Va->item) {
   case zVARs_RECDATA_: {
     Logical zOp = (Va->item == zVARs_RECDATA_), zVar;
     struct VarStruct *Var; struct CDFstruct *CDF;
-    Int32 recNum, varNt; Byte *tBuffer; int varX;
+    Int32 recNum, varNt; Byte1 *tBuffer; int varX;
     long nVars = va_arg (Va->ap, long);
     long *varNs = va_arg (Va->ap, long *);
     void *buffer = va_arg (Va->ap, char *);
