@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                       CDF `get' operations.
 *
@@ -69,7 +74,7 @@ switch (Va->item) {
 #if defined(vms) || defined(dos)
     MakeUpperString (CDFpathT);
 #endif
-    if (!ValidCDFname(CDFpathT)) return BAD_CDF_NAME;
+/*  if (!ValidCDFname(CDFpathT)) return BAD_CDF_NAME; */
     if (!sX(FindCDF(CDFpathT,&no_append,
 		    &upper_case_ext,
 		    &version_numbers),&pStatus)) return pStatus;
@@ -390,7 +395,7 @@ switch (Va->item) {
     else {
       cTotal = MAGIC_NUMBERS_SIZE + CCR.RecordSize + CPR.RecordSize;
       uTotal = MAGIC_NUMBERS_SIZE + CCR.uSize;
-      *cPct = (long) ((100L * cTotal) / uTotal);
+      *cPct = (long) (((100.0 * cTotal) / uTotal) + 0.5);
     }
     break;
   }
@@ -1399,6 +1404,7 @@ switch (Va->item) {
     struct CDFstruct *CDF;
     void *padValue = va_arg (Va->ap, void *);
     Int32 dataType, numElems, flags; OFF_T offset;
+    Int32 version, release;
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
@@ -1438,7 +1444,14 @@ switch (Va->item) {
       }
     }
     else {
-      DefaultPadValue (dataType, numElems, padValue);
+      if (!sX(ReadCDR64(CDF->fp,CDF->CDRoffset64,
+                        CDR_VERSION,&version,
+                        CDR_RELEASE,&release,
+                        CDR_NULL),&pStatus)) return pStatus;
+      if (version*100+release < 305)
+        DefaultPadValuePre350 (dataType, numElems, padValue);
+      else
+        DefaultPadValue (dataType, numElems, padValue);
       if (!sX(ConvertBuffer(HostEncoding(),CDF->decoding,
 			    CDF->negToPosFp0,dataType,
 			    numElems,padValue,padValue),&pStatus)) {
@@ -1573,7 +1586,7 @@ switch (Va->item) {
   case zVARs_RECDATA_: {
     Logical zOp = (Va->item == zVARs_RECDATA_), zVar;
     struct VarStruct *Var; struct CDFstruct *CDF;
-    Int32 recNum, varNt; Byte *tBuffer; int varX;
+    Int32 recNum, varNt; Byte1 *tBuffer; int varX;
     long nVars = va_arg (Va->ap, long);
     long *varNs = va_arg (Va->ap, long *);
     void *buffer = va_arg (Va->ap, char *);
