@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                     CDF library miscellaneous functions, part 1.
 *
@@ -28,6 +33,9 @@
 *   V1.3e 18-Nov-97, J Love	Even more Windows NT.
 *   V2.0  08/Apr-04, M liu      Replaced VSTREAM.STATS with VSTREAM_STATS.
 *   V2.1  29-Jun-04, M Liu      Added LFS (Large File Support > 2G).
+*   V2.2  26-Oct-12, M Liu      Change CDF version/release/increment and
+*                               copyright only when a newer version apps 
+*                               is applying the modification.
 *
 ******************************************************************************/
 
@@ -255,6 +263,7 @@ CDFstatus *pStatus;     /* Returned status. */
      Int32 versionNew = CDF_LIBRARY_VERSION,
            releaseNew = CDF_LIBRARY_RELEASE,
            incrementNew = CDF_LIBRARY_INCREMENT;
+     Int32 versionCur, releaseCur, incrementCur;
      uInt32 magicNumber1 = V3magicNUMBER_1,
             magicNumber2 = V3magicNUMBER_2u;
      char copyRight[CDF_COPYRIGHT_LEN+1];
@@ -342,29 +351,38 @@ CDFstatus *pStatus;     /* Returned status. */
        AbortAccess64 (CDF, noUPDATE, noDELETE);
        return FALSE;
      }
-     /*************************************************************************
-     * Update version/release/increment.
-     *************************************************************************/
      if (isLFS(CDF)) {
-       if (!sX(WriteCDR64(CDF->fp,V3_CDR_OFFSET64,
-                          CDR_VERSION,&versionNew,
-                          CDR_RELEASE,&releaseNew,
-                          CDR_INCREMENT,&incrementNew,
-                          CDR_NULL),pStatus)) {
+       /*********************************************************************
+       * Update version/release/increment and copywright only if a newer
+       * library version is modifying the CDF.
+       *********************************************************************/
+       if (!sX(ReadCDR64(CDF->fp,V3_CDR_OFFSET64,
+                         CDR_VERSION,&versionCur,
+                         CDR_RELEASE,&releaseCur,
+                         CDR_INCREMENT,&incrementCur,
+                         CDR_NULL),pStatus)) {
          AbortAccess64 (CDF, noUPDATE, noDELETE);
          return FALSE;
        }
-     }
-     /*************************************************************************
-     * Update copyright.
-     *************************************************************************/
-     CDFcopyRight (copyRight);
-     NulPad (copyRight, CDF_COPYRIGHT_LEN);
-     if (!sX(WriteCDR64(CDF->fp,V3_CDR_OFFSET64,
-                        CDR_COPYRIGHT,copyRight,
-                        CDR_NULL),pStatus)) {
-       AbortAccess64 (CDF, noUPDATE, noDELETE);
-       return FALSE;
+       if ((versionCur * 10000 + releaseCur * 100 + incrementCur) < 
+           (versionNew * 10000 + releaseNew * 100 + incrementNew)) {
+         if (!sX(WriteCDR64(CDF->fp,V3_CDR_OFFSET64,
+                            CDR_VERSION,&versionNew,
+                            CDR_RELEASE,&releaseNew,
+                            CDR_INCREMENT,&incrementNew,
+                            CDR_NULL),pStatus)) {
+           AbortAccess64 (CDF, noUPDATE, noDELETE);
+           return FALSE;
+         }
+         CDFcopyRight (copyRight);
+         NulPad (copyRight, CDF_COPYRIGHT_LEN);
+         if (!sX(WriteCDR64(CDF->fp,V3_CDR_OFFSET64,
+                            CDR_COPYRIGHT,copyRight,
+                            CDR_NULL),pStatus)) {
+           AbortAccess64 (CDF, noUPDATE, noDELETE);
+           return FALSE;
+         }
+       }
      }
    }
    /***************************************************************************
@@ -389,9 +407,9 @@ Int32 numElems;
 void *buffer;
 {
   CDFstatus pStatus = CDF_OK;
-  size_t nElemBytes = CDFelemSize(dataType);
+  size_t nElemBytes = (size_t) CDFelemSize(dataType);
   size_t nBytes = (size_t) (numElems * nElemBytes);
-  double eValue; Int32 elemN; Byte *ptr; void *tBuffer;
+  double eValue; Int32 elemN; Byte1 *ptr; void *tBuffer;
   /****************************************************************************
   * Try to encode/write entire buffer.
   ****************************************************************************/
@@ -1111,8 +1129,10 @@ Logical deleteCDF;      /* Delete CDF file(s)? */
     /**************************************************************************
     * Close all of the CDF files if they are not already closed (or deleted).
     **************************************************************************/
-    if (CDF->dotFp != NULL) V_close64 (CDF->dotFp, CDF, NULL);
-    if (CDF->uDotFp != NULL) V_close64 (CDF->uDotFp, CDF, NULL);
+    if (CDF->dotFp != NULL) V_close64 (CDF->dotFp,
+				       ((updateCDF==UPDATE)?CDF:NULL), NULL);
+    if (CDF->uDotFp != NULL) V_close64 (CDF->uDotFp,
+				        ((updateCDF==UPDATE)?CDF:NULL), NULL);
     CloseVarFiles64 (CDF);
   }               
   /****************************************************************************
