@@ -1,3 +1,8 @@
+/******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
 /****************************************************************************** 
 *
 *  SPDF/CDF                       CDF_TIME_TT2000 utility routines for C.
@@ -608,15 +613,15 @@ double *tableo;
   int ix, iy;
   int rows = CDFgetRowsinLeapSecondsTable ();
   double **table;
-  table = (double **) malloc (sizeof (double *) * rows);
+  table = (double **) cdf_AllocateMemory (sizeof (double *) * rows, NULL);
   for (ix = 0; ix < rows; ++ix)
-    table[ix] = malloc (sizeof (double) * 6);
+    table[ix] = cdf_AllocateMemory (sizeof (double) * 6, NULL);
   CDFfillLeapSecondsTable (table);
   for (ix = 0; ix < rows; ++ix) 
     for (iy = 0; iy < 6; ++iy)
       *(((double *)tableo)+6*ix+iy) = table[ix][iy];
-  for (ix = 0; ix < rows; ++ix) free (table[ix]);
-  free (table);
+  for (ix = 0; ix < rows; ++ix) cdf_FreeMemory (table[ix], NULL);
+  cdf_FreeMemory (table, NULL);
 }
 
 
