@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                            Virtual stream file.
 *
@@ -37,6 +42,10 @@
 *                               nCACHE_BUFFER_BYTEs.
 *   V5.2  20-Jun-07, D Berger   Added initialization of variables to support
 *                               READONLYon enhancements.
+*   V5.3  18-Jun-08, M Liu      Modified the V_read64 and V_write64 to make
+*                               sure the actual read/write bytes is within
+*                               the buffer size.
+*   V5.4  12-Aug-10, M Liu      Fill 0s to each new cache in AllocateBuffer.
 *
 ******************************************************************************/
 
@@ -115,8 +124,9 @@ char *a_mode;
 #if defined(vms)                
   char mrs[10+1];       /* Maximum record size. */
   char deq[10+1];       /* Default allocation quantity. */
-  sprintf (mrs, "mrs=%d", nCACHE_BUFFER_BYTEs);
-  sprintf (deq, "deq=%d", VMS_DEFAULT_nALLOCATION_BLOCKS);
+  snprintf (mrs, (size_t) sizeof(mrs), "mrs=%d", nCACHE_BUFFER_BYTEs);
+  snprintf (deq, (size_t) sizeof(deq), "deq=%d",
+	    VMS_DEFAULT_nALLOCATION_BLOCKS);
   return fopen(file_spec,a_mode,"rfm=fix",mrs,deq);
 #else
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
@@ -239,14 +249,14 @@ vFILE *vFp;
   * reached.
   ****************************************************************************/
   if (vFp->nBuffers < vFp->maxBuffers) {
-    cache = (vCACHE *) cdf_AllocateMemory (sizeof(vCACHE), NULL);
+    cache = (vCACHE *) cdf_AllocateMemory ((size_t)sizeof(vCACHE), NULL);
     if (cache != NULL) {
 #if defined(MICROSOFTC_700) && INCLUDEvMEMORY
       if (useVmem)
 	cache->ptr = (void *) AllocateVMemory (nCACHE_BUFFER_BYTEs);
       else
 #endif
-	cache->ptr = cdf_AllocateMemory (nCACHE_BUFFER_BYTEs, NULL);
+	cache->ptr = cdf_AllocateMemory ((size_t)nCACHE_BUFFER_BYTEs, NULL);
       if (cache->ptr != NULL) {
 	if (vFp->cacheHead == NULL) {
 	  vFp->cacheHead = cache;
@@ -261,6 +271,7 @@ vFILE *vFp;
 	  cache->prev = NULL;
 	}
 	(vFp->nBuffers)++;
+	memset(cache->ptr, 0, nCACHE_BUFFER_BYTEs);
 	return cache;
       }
       else {
@@ -320,6 +331,7 @@ vFILE *vFp;
     cache->prev = NULL;
   }
   (vFp->nPageOuts)++;
+  memset(cache->ptr, 0, nCACHE_BUFFER_BYTEs);
   return cache;
 }
 
@@ -352,7 +364,7 @@ long toBlockN;
 	cache->modified = FALSE;
       }
       else {
-	Byte buffer[nCACHE_BUFFER_BYTEs];
+	Byte1 buffer[nCACHE_BUFFER_BYTEs];
 	if (!vRead(((OFF_T)lastPhyBlockN) * nCACHE_BUFFER_BYTEs,
 		   buffer,(size_t)nBytes,vFp)) return FALSE;
 #if CLEAR_BYTES
@@ -377,7 +389,7 @@ long toBlockN;
        cache->modified = FALSE;
      }
      else {
-       Byte buffer[nCACHE_BUFFER_BYTEs];
+       Byte1 buffer[nCACHE_BUFFER_BYTEs];
 #if CLEAR_BYTES
        ClearBytes (buffer, 0, nCACHE_BUFFER_BYTEs - 1);
 #endif
@@ -537,12 +549,13 @@ vFILE *vFp;
 #endif
     long i; char *tmpPath; size_t pathLength;
     pathLength = strlen(vFp->path) + 1 + 8 + 1 + EXT_LEN;
-    tmpPath = (char *) cdf_AllocateMemory (pathLength + 1, NULL);
+    tmpPath = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
     if (tmpPath == NULL) return FALSE;
     for (i = 1; i <= MAX_TMP; i++) {
        strcpyX (tmpPath, vFp->path, 0);
        AppendToDir (tmpPath, "");
-       sprintf (EofS(tmpPath), "TMP%05ld.%s", i, vFp->scratchExt);
+       snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                 "TMP%05ld.%s", i, vFp->scratchExt);
        if (!IsReg(tmpPath)) {
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
          fh = OpenFile (tmpPath, WRITE_PLUS_a_mode);
@@ -652,9 +665,10 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 VISIBLE_PREFIX size_t V_read64 (buffer, item_size, n_items, vFp)
 void *buffer;           /* Pointer to buffer. */
 size_t item_size;       /* Size (in bytes) of each item to read. */
-size_t n_items;          /* Number of items to read. */
+size_t n_items;         /* Number of items to read. */
 vFILE *vFp;             /* Pointer to vFILE structure. */
 {
+  size_t nBytesX;       /* Total number of bytes in buffer. */
   size_t nBytes;	/* Total number of bytes to read. */
   OFF_T  remainingItems;	/* Number of items remaining after the offset. */
   size_t nItems;	/* Number of items to actually be read. */
@@ -666,7 +680,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   long blockN;         	/* Block number in file (from 0). */
   long atBlockN;       	/* Block number in file (from 0) at which to read. */
   vCACHE *cache;        /* Pointer to cache structure. */
-  Byte *cBuffer;        /* Pointer to cache buffer. */
+  Byte1 *cBuffer;       /* Pointer to cache buffer. */
   int remainingBytes;	/* Number of bytes remaining in a block. */
   /****************************************************************************
   * Validate read.
@@ -692,6 +706,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   else
     nItems = n_items;
   nBytes = nItems * item_size;
+  nBytesX = n_items * item_size;
   (vFp->nV_reads)++;
   /****************************************************************************
   * Read from first block...
@@ -700,6 +715,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   bufferOffset = (int) ((OFF_T) vFp->offset64 % nCACHE_BUFFER_BYTEs);
   remainingBytes = nCACHE_BUFFER_BYTEs - bufferOffset;
   xBytes = MINIMUM (nBytes, (size_t) remainingBytes);
+  if (xBytes > nBytesX) return 0;
   if (bufferOffset > 0 || xBytes < nCACHE_BUFFER_BYTEs) {
     cache = FindCache (vFp, firstBlockN);
     if (cache == NULL) cache = PageIn (vFp, firstBlockN);
@@ -713,7 +729,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
       return 0;
     }
     memmove (buffer, cBuffer + bufferOffset, (size_t) xBytes);
-    buffer = (Byte *) buffer + xBytes;
+    buffer = (Byte1 *) buffer + xBytes;
     atBlockN = firstBlockN + 1;
   }
   else
@@ -732,6 +748,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
      xBytes = (size_t) (((OFF_T)vFp->offset64) + (OFF_T) nBytes - 
 		        ((OFF_T) blockN) * nCACHE_BUFFER_BYTEs);
      xBytes = MINIMUM (xBytes, nCACHE_BUFFER_BYTEs);
+     if (xBytes > nBytesX) return 0;
      cache = FindCache (vFp, blockN);
      if (cache != NULL) {
        cBuffer = CACHEbufferREADfrom (cache);
@@ -763,7 +780,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 	 }
        }
      }
-     buffer = (Byte *) buffer + xBytes;
+     buffer = (Byte1 *) buffer + xBytes;
   }
   /****************************************************************************
   * Increment current file offset or set to EOF if the EOF indicator was set.
@@ -782,6 +799,7 @@ size_t item_size;       /* Size (in bytes) of each item to write. */
 size_t n_items;         /* Number of items to write. */
 vFILE *vFp;             /* Pointer to vFILE structure. */
 {
+  size_t nBytesX;       /* Total number of bytes in buffer. */
   size_t nBytes;        /* Total number of bytes in write. */
   long firstBlockN;     /* First block involved in write. */
   long lastBlockN;      /* Last block involved in write. */
@@ -790,7 +808,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   long atBlockN;        /* Block number in file (from 0) at which to write. */
   size_t xBytes;        /* Number of bytes in a transfer. */
   vCACHE *cache;        /* Pointer to cache structure. */
-  Byte *cBuffer;        /* Pointer to cache buffer. */
+  Byte1 *cBuffer;       /* Pointer to cache buffer. */
   size_t nBytesInBlock;	/* Number of bytes to the end of the block. */
   /****************************************************************************
   * Validate write.
@@ -804,6 +822,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   if ((int) item_size <= 0 || (int) n_items <= 0) return 0;
   vFp->eof = FALSE;		/* Cleared before proceeding. */
   nBytes = item_size * n_items;
+  nBytesX = nBytes;
   if (nBytes < 1) return 0;
   (vFp->nV_writes)++;
   /****************************************************************************
@@ -820,6 +839,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   bufferOffset = (int) ((OFF_T) vFp->offset64 % nCACHE_BUFFER_BYTEs);
   nBytesInBlock = nCACHE_BUFFER_BYTEs - bufferOffset;
   xBytes = MINIMUM (nBytes, nBytesInBlock);
+  if (xBytes > nBytesX) return 0;
   if (vFp->scratch || bufferOffset > 0 || xBytes < nCACHE_BUFFER_BYTEs) {
     cache = FindCache (vFp, firstBlockN);
     if (cache == NULL) {
@@ -858,7 +878,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
     cache->modified = TRUE;
     vFp->length64 = MaxLongLong (vFp->length64, vFp->offset64 + 
 				  (OFF_T) xBytes);
-    buffer = (Byte *) buffer + xBytes;
+    buffer = (Byte1 *) buffer + xBytes;
     atBlockN = firstBlockN + 1;
   }
   else
@@ -877,6 +897,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
      xBytes = (size_t) (((OFF_T)vFp->offset64) + (OFF_T) nBytes - 
 		        ((OFF_T) blockN) * nCACHE_BUFFER_BYTEs);
      xBytes = MINIMUM (xBytes, nCACHE_BUFFER_BYTEs);
+     if (xBytes > nBytesX) return 0;
      /*************************************************************************
      * Is this block in the cache?  If so, move the number of bytes to be
      * written at this block to its cache buffer.
@@ -941,7 +962,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
      }
      vFp->length64 = MaxLongLong (vFp->length64, ((OFF_T) blockN) *
 			          nCACHE_BUFFER_BYTEs + (OFF_T) xBytes);
-     buffer = (Byte *) buffer + xBytes;
+     buffer = (Byte1 *) buffer + xBytes;
   }
   /****************************************************************************
   * Increment current file offset.
@@ -1006,7 +1027,8 @@ char *a_mode;           /* Access mode. */
 	FCLOSE64 (fp);
 	return NULL;
       }
-      for (i = 0; i < nCACHE_BUFFER_BYTEs; i++) buffer[i] = 0;
+      /* for (i = 0; i < nCACHE_BUFFER_BYTEs; i++) buffer[i] = 0; */
+      memset (buffer, 0, (size_t) nCACHE_BUFFER_BYTEs);
       if (FREAD64(buffer,EOFoffsetInBlock,1,fp) != 1) {
 	FCLOSE64 (fp);
 	return NULL;
@@ -1031,7 +1053,7 @@ char *a_mode;           /* Access mode. */
   /****************************************************************************
   * Allocate and load vFILE structure.
   ****************************************************************************/
-  vFp = (vFILE *) cdf_AllocateMemory (sizeof(vFILE), NULL);
+  vFp = (vFILE *) cdf_AllocateMemory ((size_t)sizeof(vFILE), NULL);
   if (vFp == NULL) {
     FCLOSE64 (fp);
     return NULL;
@@ -1042,7 +1064,7 @@ char *a_mode;           /* Access mode. */
 #else
   vFp->fp = fp;
 #endif
-  vFp->path = (char *) cdf_AllocateMemory (strlen(file_spec) + 1, NULL);
+  vFp->path = (char *) cdf_AllocateMemory ((size_t)strlen(file_spec) + 1, NULL);
   if (vFp->path == NULL) {
     cdf_FreeMemory (vFp, NULL);
     FCLOSE64 (fp);
