@@ -1,4 +1,9 @@
 	PROGRAM QST2IF
+C-----------------------------------------------------------------------
+C Copyright 1996-2014 United States Government as represented by the
+C Administrator of the National Aeronautics and Space Administration.
+C All Rights Reserved.
+C-----------------------------------------------------------------------
 
 C-----------------------------------------------------------------------
 C
@@ -83,7 +88,7 @@ C-----------------------------------------------------------------------
         COMMON /BLK1/recBuffer1,recBuffer2
         COMMON /BLK2/recBuffer1OUT,recBuffer2OUT
 
-	DATA ENCODING/NETWORK_ENCODING/, MAJORITY/COL_MAJOR/,
+	DATA ENCODING/IBMPC_ENCODING/, MAJORITY/COL_MAJOR/,
      .	     NUM_DIMS/2/, DIM_SIZES/2,3/,
      .	     VAR_DATA_TYPE/CDF_CHAR/, VAR_NUM_ELEMENTS/5/,
      .	     VAR_REC_VARIANCE/VARY/, VAR_DIM_VARIANCES/VARY,VARY/,
@@ -178,8 +183,8 @@ C-----------------------------------------------------------------------
         CALL CDF_create_cdf (CDFNAME, ID, STATUS)
         IF (STATUS .LT. CDF_OK) CALL QUIT_CDF (STATUS, '6.0')
 
-        CALL CDF_set_encoding (ID, ENCODING, STATUS)
-        IF (STATUS .LT. CDF_OK) CALL QUIT_CDF (STATUS, '6.1')
+C       CALL CDF_set_encoding (ID, ENCODING, STATUS)
+C       IF (STATUS .LT. CDF_OK) CALL QUIT_CDF (STATUS, '6.1')
 
         CALL CDF_set_majority (ID, MAJORITY, STATUS)
         IF (STATUS .LT. CDF_OK) CALL QUIT_CDF (STATUS, '6.2')
@@ -209,7 +214,7 @@ C-----------------------------------------------------------------------
         CALL CDF_get_cachesize (ID, cacheSizeOUT, STATUS)
         IF (STATUS .LT. CDF_OK) CALL QUIT_CDF (STATUS, '7.7')
 
-	IF (ENCODING_OUT .NE. ENCODING) CALL QUIT_CDF (STATUS, '7.10')
+C	IF (ENCODING_OUT .NE. ENCODING) CALL QUIT_CDF (STATUS, '7.10')
 	IF (MAJORITY_OUT .NE. MAJORITY) CALL QUIT_CDF (STATUS, '7.11')
 	IF (NUM_ATTRS_OUT .NE. 0) CALL QUIT_CDF (STATUS, '7.14')
 	IF (cacheSizeOUT .NE. cacheSize) CALL QUIT_CDF (STATUS, '7.15')
@@ -510,7 +515,7 @@ C-----------------------------------------------------------------------
 	  END DO
 	END IF
 
-	IF (ENCODING_OUT .NE. ENCODING) CALL QUIT_CDF (STATUS, '22.4')
+C	IF (ENCODING_OUT .NE. ENCODING) CALL QUIT_CDF (STATUS, '22.4')
 	IF (MAJORITY_OUT .NE. MAJORITY) CALL QUIT_CDF (STATUS, '22.5')
 	IF (MAX_ZREC_OUT .NE. 11) CALL QUIT_CDF (STATUS, '22.6')
 	IF (NUM_ZVARS_OUT .NE. 3) CALL QUIT_CDF (STATUS, '22.7')
