@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                    CDFedit, auxiliary routines.
 *
@@ -18,6 +23,7 @@
 *   V1.4  26-Aug-96, J Love     CDF V2.6.
 *   V1.4a 21-Feb-97, J Love	Removed RICE.
 *   V1.4b 14-Dec-97, J Love	Added ALPHAVMSi encoding.
+*   V3.3  10-Apr-09, M Liu      Changed "MAC" to "PPC" for encoding.
 *
 ******************************************************************************/
 
@@ -151,7 +157,7 @@ va_dcl
        EW.nColsTotal = MaxInt (EW.nColsTotal, (int) (length + 2));
        textLength += length + 1;
     }
-    EW.eText = (char *) cdf_AllocateMemory (textLength + 1, FatalError);
+    EW.eText = (char *) cdf_AllocateMemory ((size_t)textLength + 1, FatalError);
     MakeNUL (EW.eText);
     for (lineN = 0; !NULstring(lineS[lineN]); lineN++) {
        strcatX (EW.eText, lineS[lineN], textLength);
@@ -407,7 +413,8 @@ Logical attr;   /* TRUE: attribute, FALSE: variable. */
   AOSs6 (lineS, " ",BLANKs78," ","  Enter any key to continue...  "," ","")
   size_t length = strlen (name);
   char delim = PickDelimiter (name, length);
-  sprintf (lineS[1], "  %c%s%c  ", delim, name, delim);
+  snprintf (lineS[1], (size_t) sizeof(BLANKs78),
+	    "  %c%s%c  ", delim, name, delim);
   MessageWindow (lineS, BOO(attr,"Full attribute name...",
 				 "Full variable name..."), LogicalTRUE);
   MessageWindow (NULL);
@@ -516,7 +523,7 @@ Logical SelectEncoding () {
   AOSs1 (trailer, "Select: ________   Exit: ________   Help:   ________")
   static char *items[] = {
     "HOST      SUN   IBMPC   SGi   HP     ALPHAVMSg   ALPHAVMSi   DECSTATION",
-    "NETWORK   VAX   IBMRS   MAC   NeXT   ALPHAVMSd   ALPHAOSF1"
+    "NETWORK   VAX   IBMRS   PPC   NeXT   ALPHAVMSd   ALPHAOSF1"
   };
   static int iLineNs[] = { 0,0,0,0,0,0,0,0,1,1,1,1,1,1,1 };
   static int iCols[] = { 0,10,16,24,30,37,49,61,0,10,16,24,30,37,49 };
@@ -767,11 +774,11 @@ char *label;            /* If NULL, no label. */
   static int iLens[] = { DTiwLENs };
   static int exitChars[] = { ENTERkey_FSI, EXITkey_FSI, HELPkey_FSI, NUL };
   static struct ItemWindowStruct IWt = {
-    4, 1, 78, NULL, 1, header, 3, items, 15, iLineNs, iCols, iLens, 3, 1,
+    4, 1, 78, NULL, 1, header, 3, items, 17, iLineNs, iCols, iLens, 3, 1,
     trailer, exitChars, REFRESHkey_FSI, FALSE
   };
   static Logical first = TRUE; int itemN;
-  char selectedDataType[11+1], delim = ' ', *selected = NULL, *tmpStr;
+  char selectedDataType[15+1], delim = ' ', *selected = NULL, *tmpStr;
   if (first) {
     EncodeKeyDefinitions (1, trailer, ENTERkey_FSI, EXITkey_FSI,
 			  HELPkey_FSI);
