@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                            CDFstats (statistics).
 *
@@ -28,6 +33,7 @@
 *   V2.5   8-Aug-96, J Love	CDF V2.6.
 *   V2.5a 21-Feb-97, J Love	Removed RICE.
 *   V2.5b 18-Nov-97, J Love	Windows NT/Visual C++.
+*   V3.3  10-Jan-09, M Liu      Validate a file before it is used.
 *
 ******************************************************************************/
 
@@ -55,7 +61,6 @@ Logical report[3];
 #if !defined(win32) || (defined(win32) && defined(ALONE))
 MAIN {
   Logical success = TRUE;
-int i;
   strcpyX (pgmName, "CDFstats", MAX_PROGRAM_NAME_LEN);
 #if defined(mac)
   MacExecuteSO (CalculateStatistics,StatisticsQOPs);
@@ -102,6 +107,8 @@ char *argV[];
     FALSE, FALSE, FALSE, 0
   };
   static char *reportTokens[] = { "errors", "warnings", "informationals" };
+
+  CDFsetValidate (VALIDATEFILEon);
   /****************************************************************************
   * Get qualifiers/options/parameters.
   ****************************************************************************/
@@ -112,7 +119,7 @@ char *argV[];
     case 2:
       if (strcmp(argV[1],"-java") == 0) {
         pagingOn = FALSE;
-        PageOLH ("cdfstatj.olh", argV[0]);
+        PageOLH ("cdfstatsj.olh", argV[0]);
         return TRUE;
       }
     default:
@@ -418,24 +425,28 @@ struct VarStruct *Var;
   ****************************************************************************/
   strcpyX (line, "\n", MAX_SCREENLINE_LEN);
   delim = PickDelimiter (Var->varName, strlen(Var->varName));
-  sprintf (EofS(line), "%3ld.  %c%s%c",
-	   Var->varN + 1, delim, Var->varName, delim);
+  snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
+	    "%3ld.  %c%s%c", Var->varN + 1, delim, Var->varName, delim);
   if (Var->Z) {
-    sprintf (EofS(line), "   %ld:[", Var->numDims);
+    snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
+	      "   %ld:[", Var->numDims);
     if (Var->numDims > 0) {
       for (dimN = 0; dimN < Var->numDims; dimN++) {
 	 if (dimN > 0) strcatX (line, ",", MAX_SCREENLINE_LEN);
-	 sprintf (EofS(line), "%ld", Var->dimSizes[dimN]);
+	 snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
+		   "%ld", Var->dimSizes[dimN]);
       }
     }
     strcatX (line, "]", MAX_SCREENLINE_LEN);
   }
-  sprintf (EofS(line), "   %s/", TFvarianceToken(Var->recVary));
+  snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
+	    "   %s/", TFvarianceToken(Var->recVary));
   for (dimN = 0; dimN < Var->numDims; dimN++) {
-     sprintf (EofS(line), "%s", TFvarianceToken(Var->dimVarys[dimN]));
+     snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
+	       "%s", TFvarianceToken(Var->dimVarys[dimN]));
   }
-  sprintf (EofS(line), "   (CDF_%s/%ld)",
-	   DataTypeToken(Var->dataTypeV), Var->numElemsV);
+  snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
+	    "   (CDF_%s/%ld)", DataTypeToken(Var->dataTypeV), Var->numElemsV);
   strcatX (line, "\n\n", MAX_SCREENLINE_LEN);
   WriteOut (OUTfp, line);
   /****************************************************************************
@@ -776,7 +787,7 @@ char **argV[];
         *argC = 14 + BOO(NULpString(CDFtext),0,1) +
 		     BOO(NULpString(outText),0,2) +
 		     BOO(NULpString(cacheText),0,2);
-        *argV = (char **) cdf_AllocateMemory (*argC * sizeof(char *),
+        *argV = (char **) cdf_AllocateMemory ((size_t)*argC * sizeof(char *),
 					  FatalError);
 		n = 0;
 		MAKEstrARGv (argV, n, pgmName)
@@ -798,7 +809,7 @@ char **argV[];
 				     "-noupdate_monotonic")
 		MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
 		MAKEstrARGv (argV, n, "-zmode")
-		sprintf (tempS1, "%d", zMode);
+		snprintf (tempS1, (size_t) sizeof(tempS1), "%d", zMode);
 		MAKEstrARGv (argV, n, tempS1)
 		MAKEstrARGv (argV, n, "-report")
 		MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
@@ -832,7 +843,7 @@ char **argV[];
       case HELPin: {
         int n;
         *argC = 1;
-        *argV = (char **) cdf_AllocateMemory (*argC * sizeof(char *), FatalError);
+        *argV = (char **) cdf_AllocateMemory ((size_t)*argC * sizeof(char *), FatalError);
 		n = 0;
 		MAKEstrARGv (argV, n, pgmName)
 		CloseDialog (dialogP);
